=== modified file 'bzrlib/branch.py'
--- bzrlib/branch.py	2005-10-03 02:10:02 +0000
+++ bzrlib/branch.py	2005-10-03 02:54:29 +0000
@@ -991,6 +991,10 @@
         """Return a `Tree` for the working copy."""
         from bzrlib.workingtree import WorkingTree
         # TODO: In the future, WorkingTree should utilize Transport
+        # RobertCollins 20051003 - I don't think it should - working trees are
+        # much more complex to keep consistent than our careful .bzr subset.
+        # instead, we should say that working trees are local only, and optimise
+        # for that.
         return WorkingTree(self._transport.base, self.read_working_inventory())
 
 

=== modified file 'bzrlib/builtins.py'
--- bzrlib/builtins.py	2005-10-03 01:42:16 +0000
+++ bzrlib/builtins.py	2005-10-03 02:54:29 +0000
@@ -796,16 +796,9 @@
             tree = b.working_tree()
         else:
             tree = b.revision_tree(revision.in_history(b).rev_id)
-
-        for fp, fc, kind, fid in tree.list_files():
+        for fp, fc, kind, fid, entry in tree.list_files():
             if verbose:
-                if kind == 'directory':
-                    kindch = '/'
-                elif kind == 'file':
-                    kindch = ''
-                else:
-                    kindch = '???'
-
+                kindch = entry.kind_character()
                 print '%-8s %s%s' % (fc, fp, kindch)
             else:
                 print fp
@@ -885,7 +878,7 @@
     See also: bzr ignore"""
     def run(self):
         tree = Branch.open_containing('.').working_tree()
-        for path, file_class, kind, file_id in tree.list_files():
+        for path, file_class, kind, file_id, entry in tree.list_files():
             if file_class != 'I':
                 continue
             ## XXX: Slightly inefficient since this was already calculated

=== modified file 'bzrlib/inventory.py'
--- bzrlib/inventory.py	2005-10-03 02:10:02 +0000
+++ bzrlib/inventory.py	2005-10-03 02:54:29 +0000
@@ -160,6 +160,16 @@
         else:
             raise BzrError("unhandled entry kind %r" % kind)
 
+    def kind_character(self):
+        """Return a short kind indicator useful for appending to names."""
+        if self.kind == 'directory':
+            return '/'
+        if self.kind == 'file':
+            return ''
+        if self.kind == 'symlink':
+            return ''
+        raise RuntimeError('unreachable code')
+
     def read_symlink_target(self, path):
         if self.kind == 'symlink':
             try:

=== modified file 'bzrlib/selftest/__init__.py'
--- bzrlib/selftest/__init__.py	2005-10-02 21:51:29 +0000
+++ bzrlib/selftest/__init__.py	2005-10-03 02:54:29 +0000
@@ -463,6 +463,7 @@
                    'bzrlib.selftest.blackbox',
                    'bzrlib.selftest.testtransport',
                    'bzrlib.selftest.testgraph',
+                   'bzrlib.selftest.testworkingtree',
                    ]
 
     for m in (bzrlib.store, bzrlib.inventory, bzrlib.branch,

=== modified file 'bzrlib/selftest/testinv.py'
--- bzrlib/selftest/testinv.py	2005-09-23 09:25:16 +0000
+++ bzrlib/selftest/testinv.py	2005-10-03 02:54:29 +0000
@@ -15,9 +15,9 @@
 # Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 
 import os
-from bzrlib.selftest import TestCase
+from bzrlib.selftest import TestCase, TestCaseInTempDir
 
-from bzrlib.inventory import Inventory, InventoryEntry
+from bzrlib.inventory import Inventory, InventoryEntry, ROOT_ID
 
 
 class TestInventory(TestCase):
@@ -59,3 +59,17 @@
         ie = inv.add_path('foo.txt', 'file')
         ## XXX
 
+
+class TestInventoryEntry(TestCaseInTempDir):
+
+    def test_file_kind_character(self):
+        file = InventoryEntry('123', 'hello.c', 'file', ROOT_ID)
+        self.assertEqual(file.kind_character(), '')
+
+    def test_dir_kind_character(self):
+        dir = InventoryEntry('123', 'hello.c', 'directory', ROOT_ID)
+        self.assertEqual(dir.kind_character(), '/')
+
+    def test_link_kind_character(self):
+        dir = InventoryEntry('123', 'hello.c', 'symlink', ROOT_ID)
+        self.assertEqual(dir.kind_character(), '')

=== modified file 'bzrlib/tree.py'
--- bzrlib/tree.py	2005-10-03 01:42:16 +0000
+++ bzrlib/tree.py	2005-10-03 02:54:29 +0000
@@ -161,12 +161,13 @@
     def list_files(self):
         # The only files returned by this are those from the version
         for path, entry in self.inventory.iter_entries():
-            yield path, 'V', entry.kind, entry.file_id
+            yield path, 'V', entry.kind, entry.file_id, entry
 
     def get_symlink_target(self, file_id):
         ie = self._inventory[file_id]
         return ie.symlink_target;
 
+
 class EmptyTree(Tree):
     def __init__(self):
         self._inventory = Inventory()
@@ -178,8 +179,7 @@
         return False
 
     def list_files(self):
-        if False:  # just to make it a generator
-            yield None
+        return iter([])
     
     def __contains__(self, file_id):
         return file_id in self._inventory
@@ -189,8 +189,6 @@
         return None
 
 
-
-
 ######################################################################
 # diff
 

=== modified file 'bzrlib/workingtree.py'
--- bzrlib/workingtree.py	2005-10-03 01:42:16 +0000
+++ bzrlib/workingtree.py	2005-10-03 02:54:29 +0000
@@ -29,6 +29,58 @@
 from bzrlib.errors import BzrCheckError
 from bzrlib.trace import mutter
 
+class TreeEntry(object):
+    """An entry that implements the minium interface used by commands.
+
+    This needs further inspection, it may be better to have 
+    InventoryEntries without ids - though that seems wrong. For now,
+    this is a parallel hierarchy to InventoryEntry, and needs to become
+    one of several things: decorates to that hierarchy, children of, or
+    parents of it.
+    """
+ 
+    def __eq__(self, other):
+        # yes, this us ugly, TODO: best practice __eq__ style.
+        return (isinstance(other, TreeEntry)
+                and other.__class__ == self.__class__)
+ 
+    def kind_character(self):
+        return "???"
+
+
+class TreeDirectory(TreeEntry):
+    """See TreeEntry. This is a directory in a working tree."""
+
+    def __eq__(self, other):
+        return (isinstance(other, TreeDirectory)
+                and other.__class__ == self.__class__)
+
+    def kind_character(self):
+        return "/"
+
+
+class TreeFile(TreeEntry):
+    """See TreeEntry. This is a regular file in a working tree."""
+
+    def __eq__(self, other):
+        return (isinstance(other, TreeFile)
+                and other.__class__ == self.__class__)
+
+    def kind_character(self):
+        return ''
+
+
+class TreeLink(TreeEntry):
+    """See TreeEntry. This is a symlink in a working tree."""
+
+    def __eq__(self, other):
+        return (isinstance(other, TreeLink)
+                and other.__class__ == self.__class__)
+
+    def kind_character(self):
+        return ''
+
+
 class WorkingTree(bzrlib.tree.Tree):
     """Working copy tree.
 
@@ -185,7 +237,20 @@
                                             "now of kind %r"
                                             % (fap, f_ie.kind, f_ie.file_id, fk))
 
-                yield fp, c, fk, (f_ie and f_ie.file_id)
+                # make a last minute entry
+                if f_ie:
+                    entry = f_ie
+                else:
+                    if fk == 'directory':
+                        entry = TreeDirectory()
+                    elif fk == 'file':
+                        entry = TreeFile()
+                    elif fk == 'symlink':
+                        entry = TreeLink()
+                    else:
+                        entry = TreeEntry()
+                
+                yield fp, c, fk, (f_ie and f_ie.file_id), entry
 
                 if fk != 'directory':
                     continue

